package com.bpcl.lpg.core.dao.impl;

import de.hybris.platform.servicelayer.exceptions.ModelNotFoundException;
import de.hybris.platform.servicelayer.exceptions.AmbiguousIdentifierException;
import java.util.Map;
import java.util.HashMap;
import de.hybris.platform.servicelayer.util.ServicesUtil;
import com.bpcl.lpg.core.model.LPGDistributorStaffModel;
import com.bpcl.lpg.core.model.BpclEkycTransactionModel;
import com.bpcl.core.model.UserOTPModel;
import java.util.Collection;
import org.apache.commons.collections.CollectionUtils;
import com.bpcl.core.enums.OTPStatus;
import com.bpcl.core.enums.OTPChannel;
import com.bpcl.core.enums.OTPType;
import org.apache.commons.lang.time.DateUtils;
import java.util.Date;
import de.hybris.platform.servicelayer.search.SearchResult;
import de.hybris.platform.servicelayer.search.FlexibleSearchQuery;
import com.bpcl.lpg.core.model.BpclEkycConsentsModel;
import java.util.List;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import javax.annotation.Resource;
import de.hybris.platform.servicelayer.search.FlexibleSearchService;
import com.bpcl.lpg.core.dao.BpclEkycDao;

public class BpclEkycDaoImpl implements BpclEkycDao
{
    private static final String GET_CONSENTS_DETAILS = "SELECT {pk} FROM {BpclEkycConsents} WHERE {partnerCode}=?partnerCode ORDER BY {sequenceNumber} ASC";
    private static final String GET_EKYC_TRXN_DETAILS = "SELECT {pk} FROM {BpclEkycTransaction} WHERE {eKycTransactionId}=?eKycTransactionId";
    private static final String GET_CONSENT_DETAILS = "SELECT {pk} FROM {BpclEkycConsents} WHERE {languageCode}=?languageCode AND {partnerCode}=?partnerCode";
    private static final String OTP_VALID_FOR_TIME = "SELECT {pk} FROM {USEROTP} WHERE {customerid} = ?customerId AND {OTPTYPE} = ?otpType AND {OTPCHANNEL} = ?channel AND {OTPSTATUS} = ?otpStatus AND ?currentDateTIme > {lastverifiedtime} AND {lastverifiedtime} > ?thresholdTime";
    private static final String OPERATOR_BY_MOBILENUMBER_QUERY = "SELECT {pk} FROM {LPGDistributorStaff} WHERE {uid}=?uid";
    private static final int ZERO = 0;
    @Resource
    private FlexibleSearchService flexibleSearchService;
    @Resource
    private ConfigurationService configurationService;
    
    public List<BpclEkycConsentsModel> getAllBpclEkycConsents(final String partnerCode) {
        final FlexibleSearchQuery query = new FlexibleSearchQuery("SELECT {pk} FROM {BpclEkycConsents} WHERE {partnerCode}=?partnerCode ORDER BY {sequenceNumber} ASC");
        query.addQueryParameter("partnerCode", (Object)partnerCode);
        final SearchResult<BpclEkycConsentsModel> consentsModel = (SearchResult<BpclEkycConsentsModel>)this.flexibleSearchService.search(query);
        return consentsModel.getResult();
    }
    
    public boolean isOTPAuthSuccess(final String customerId) {
        final FlexibleSearchQuery query = new FlexibleSearchQuery("SELECT {pk} FROM {USEROTP} WHERE {customerid} = ?customerId AND {OTPTYPE} = ?otpType AND {OTPCHANNEL} = ?channel AND {OTPSTATUS} = ?otpStatus AND ?currentDateTIme > {lastverifiedtime} AND {lastverifiedtime} > ?thresholdTime");
        query.addQueryParameter("customerId", (Object)customerId);
        query.addQueryParameter("currentDateTIme", (Object)new Date());
        query.addQueryParameter("thresholdTime", (Object)DateUtils.addMinutes(new Date(), -this.configurationService.getConfiguration().getInt("ekyc.otp.threshold.time", 30)));
        query.addQueryParameter("otpType", (Object)OTPType.EKYC);
        query.addQueryParameter("channel", (Object)OTPChannel.MOBILE);
        query.addQueryParameter("otpStatus", (Object)OTPStatus.CONSUMED);
        final SearchResult<UserOTPModel> UserOTPModel = (SearchResult<UserOTPModel>)this.flexibleSearchService.search(query);
        return CollectionUtils.isNotEmpty((Collection)UserOTPModel.getResult());
    }
    
    public BpclEkycConsentsModel getConsentByLangCode(final String languageCode, final String partnerCode) {
        final FlexibleSearchQuery query = new FlexibleSearchQuery("SELECT {pk} FROM {BpclEkycConsents} WHERE {languageCode}=?languageCode AND {partnerCode}=?partnerCode");
        query.addQueryParameter("languageCode", (Object)languageCode);
        query.addQueryParameter("partnerCode", (Object)partnerCode);
        final SearchResult<BpclEkycConsentsModel> consentsModel = (SearchResult<BpclEkycConsentsModel>)this.flexibleSearchService.search(query);
        return consentsModel.getResult().get(0);
    }
    
    public BpclEkycTransactionModel getEkycTrxnByTrxnId(final String eKycTransactionId) {
        final FlexibleSearchQuery query = new FlexibleSearchQuery("SELECT {pk} FROM {BpclEkycTransaction} WHERE {eKycTransactionId}=?eKycTransactionId");
        query.addQueryParameter("eKycTransactionId", (Object)eKycTransactionId);
        final SearchResult<BpclEkycTransactionModel> ekycTrxnModel = (SearchResult<BpclEkycTransactionModel>)this.flexibleSearchService.search(query);
        if (null != ekycTrxnModel && CollectionUtils.isNotEmpty((Collection)ekycTrxnModel.getResult())) {
            return ekycTrxnModel.getResult().get(0);
        }
        return null;
    }
    
    public LPGDistributorStaffModel findOperatorByCustomerId(final String customerId) {
        ServicesUtil.validateParameterNotNullStandardMessage("uid", (Object)customerId);
        final Map<String, String> params = new HashMap<String, String>();
        params.put("uid", customerId);
        final FlexibleSearchQuery searchQuery = new FlexibleSearchQuery("SELECT {pk} FROM {LPGDistributorStaff} WHERE {uid}=?uid");
        searchQuery.addQueryParameters((Map)params);
        final SearchResult<LPGDistributorStaffModel> searchResult = (SearchResult<LPGDistributorStaffModel>)this.flexibleSearchService.search(searchQuery);
        final List<LPGDistributorStaffModel> operatorList = searchResult.getResult();
        if (operatorList.size() > 1) {
            throw new AmbiguousIdentifierException("Found" + operatorList.size() + "Operators with the MobileNumber value:mobileNumberwhich should be unique");
        }
        if (operatorList.isEmpty()) {
            throw new ModelNotFoundException("No Operator with mobileNumber: " + customerId + " found");
        }
        return operatorList.get(0);
    }
}
